function u_n1 = diffusion_tc(H, f, m, p, q, u_n, iter, nelem, npn, Le, ua, ub)
% This function is the driver for a one-dimensional finite element code
% solving the diffusion equation given by
%
%       (p u')' - q u + f - m u_t = 0                 0 < x < 1
%
% Quadratic interpolation is used in the spatial domain as
%
%        u(s) = n1(s) u1 + n2(s) u2 +n3(s) u3    .
%
%               n1(s) = 0.5*s*(s-1)
%
%               n2(s) = (1+s)*(1-s)       -1 < s < 1
%
%               n3(s) = 0.5*s*(s+1)
%
% The Crank-Nicholson method is used to advance the solution in the time 
% domain.  Following Foslien: pg 454-56 (1994), the equations lead to  
% algebraic equations of the form
%          [K] {u_n1} = {r_n1}
%
% where u_n1 is the solution at t_n+1 and
%
%          [K] = [M]+(H/2)[A]
%
%          {r_n1} = [[M]-(H/2)[A]]{u_n} +(H/2)({f_n}+{f_n1})
%
%                    [A]= sum_e [a_e]  &  [M] = sum_e [m_e]
%                               
%                             {f} = sum_e {f_e}
%
% Only essential (Dirichlet) boundary conditions are allowed.
% 
% Consecutive nodal numbering is assumed with node 1 located at x=0.
% All elements have equal length.
%
% ********************     Variable Definitions     ********************
%
% aelem     [a_e] = [p_e] + [q_e] : element level matrices of size (3,3)
% belem     [b_e] = [m_e] - 0.5*H*[a_e]: dimension(3,3)
% Bmtrx     [B] = [M] - 0.5*H*[A]
% f         f(x,t) identified in d.e.: dimension(npoin)
% fe_n      local nodal values of f(x,t): n=1,2,3
% felem     {f_e} defined in Foslien pg 54: dimension(3)
% iter      iternation number within a time step: iter=0 on initial step
%                   in time
% H         time step size
% kelem     [k_e] = [m_e]+0.5*H*[a_e]
% Kmtrx     [K] as shown above: (npoin,npoin)
% Kmtrx_bc  [K] modified by essential boundary conditions
% Le        element length
% m         m(x,t) identified in d.e.: dimension(npoin)
% me_n      local nodal values of m(x,t): n=1,2,3
% melem     [m_e] defined in Foslien pg 54: dimension(3,3)
% nelem     number of elements
% npoin     number of nodes
% x         spatial coordinate: dimension(npoin)
% p         p(x,t) identified in d.e.: dimension(npoin)
% pe_n      local nodal values of p(x,t): n=1,2,3
% pelem     [p_e] defined in Foslien pg 54 dimension(3,3)
% q         q(x,t) identified in d.e.: dimension(npoin)
% qe_n      local nodal values of q(x,t): n=1,2,3
% qelem     [q_e] defined in Foslien pg 54 dimension(3,3)
% rload     load vector {r_n1} at time t_n1: dimension(npoin)
% u_n         solution vector at time t_n : dimension(npoin)
% u_n1      solution vector at time t_n1
% ua        boundary condition at ground level (x=0)
% ub        boundary condition at surface (x=1)--may be time dependent
% y         intermediate variable in LU solution
%
% ********************     End of variables     ********************
persistent Kmtrx Bmtrx Lmtrx Umtrx;
%
%  Update the global "stiffness" matrices on initial step forward in time
if (iter == 0);
    Kmtrx = zeros(npn);
    Bmtrx = zeros(npn);
    %
    % loop over the elements
    for ielem = 1:nelem;
        idfn = 2*(ielem-1)+1;
        %
        % Zero the element matrices
        aelem=zeros(3);
        melem=zeros(3);
        %
        % Define nodal values of m(x,t), p(xt), and q(x,t).
        me1 = m(idfn);
        me2 = m(idfn+1);
        me3 = m(idfn+2);
        %
        pe1 = p(idfn);
        pe2 = p(idfn+1);
        pe3 = p(idfn+2);
        %
        qe1 = q(idfn);
        qe2 = q(idfn+1);
        qe3 = q(idfn+2);
        %
        % Define integration functions: Bickford pg 196-7 (1990)
        p_int = @(d1,d2,d3)((d1*pe1+d2*pe2+d3*pe3)/(30*Le));
        %
        q_int = @(d1,d2,d3)((d1*qe1+d2*qe2+d3*qe3)*(Le/420));
                %
        m_int = @(d1,d2,d3)((d1*me1+d2*me2+d3*me3)*(Le/420));
        %
        % Compute the element A matrix as defined in Foslien (pg 54)
        aelem(1,1) = p_int(37,36,-3) + q_int(39,20,-3);
        aelem(1,2) = p_int(-44,-32,-4) + q_int(20,16,-8);
        aelem(1,3) = p_int(7,-4,7) + q_int(-3,-8,-3);
        %
        aelem(2,2) = p_int(48,64,48) + q_int(16,192,16);
        aelem(2,3) = p_int(-4,-32,-44) + q_int(-8,16,20);
        %
        aelem(3,3) = p_int(-3,36,37) + q_int(-3,20,39);
        %
        % Enforce symmetry
        aelem(2,1) = aelem(1,2);
        aelem(3,1) = aelem(1,3);
        aelem(3,2) = aelem(2,3);
        %
        % Compute the element m matrix as defined in Foslien (pg 54)
        melem(1,1) = m_int(39,20,-3); 
        melem(1,2) = m_int(20,16,-8); 
        melem(1,3) = m_int(-3,-8,-3); 
        %
        melem(2,2) = m_int(16,192,16); 
        melem(2,3) = m_int(-8,16,20); 
        %
        melem(3,3) = m_int(-3,20,39) ;
        %
        % Enforce symmetry
        melem(2,1) = melem(1,2);
        melem(3,1) = melem(1,3);
        melem(3,2) = melem(2,3);
        %
        % Compute the element stiffness matrix
        kelem = melem + 0.5*H*aelem;
        %
        % Compute the matrix [melem - 0.5*H*aelem] named k2elem
        belem = melem - 0.5*H*aelem;
        %
        % Assemble the element stiffness matrices into global matrices
        Kmtrx(idfn:idfn+2,idfn:idfn+2) = Kmtrx(idfn:idfn+2,idfn:idfn+2)...
            + kelem;
        Bmtrx(idfn:idfn+2,idfn:idfn+2) = Bmtrx(idfn:idfn+2,idfn:idfn+2)...
            + belem;
    end
end
% Assemble the load vector  
%
% First, initialize the load vector to zero
rload = zeros(npn,1);
%
% loop over the elements
for ielem = 1:nelem;
    idfn = 2*(ielem-1)+1;
    %                
    % Define nodal values of f(x,t). NOTE: f is defined prior to entering
    % "function diffusion" as f=0.5*(f_n+f_n1) per Crank-Nicholson
    % algorithm.
    fe1 = f(idfn);
    fe2 = f(idfn+1);
    fe3 = f(idfn+2);
    %
    f_int = @(d1,d2,d3)((d1*fe1+d2*fe2+d3*fe3)*(Le/30));
    %
    % Compute the nodal load vector
    felem(1,1) = H*f_int(4,2,-1);
    felem(2,1) = H*f_int(2,16,2);
    felem(3,1) = H*f_int(-1,2,4);
    %
    % Assemble the load vector
    % First compute the load due to f(x,t)
    rload(idfn:idfn+2) = rload(idfn:idfn+2)+ felem;
    %
end
%
% Now add the contribution of [B]{theta} (See Bickford, pg 424)
rload = rload + Bmtrx*u_n;
%
% Implement the boundary conditions
[Kmtrx_bc, rload] = bnd_conds(Kmtrx, rload, iter, npn, ua, ub);
%
% Factorize the stiffness matrix if [A] has beeen updated
% [Kmtrx_bc] = [L][U]
if (iter == 0);
    [Lmtrx,Umtrx]=lu(Kmtrx_bc);    
end
% Solve the system of equations using the factorized matrix: [A]=[L][U]
y = Lmtrx\rload;
u_n1 = Umtrx\y;   
end     % end function diffusion
%
%
function[Kmtrx_bc, rload] ...
    = bnd_conds(Kmtrx, rload, iter, npn, ua, ub)
% This function implements essential boundary conditions using matrix
% modification.
    Kmtrx_bc = Kmtrx;
    % Implement the essential b.c. u=0.
    rload(:,1) =rload(:,1) - Kmtrx(:,1)*ua;
    rload(1,1) = ua;
    Kmtrx_bc(:,1) = 0;
    Kmtrx_bc(1,:) = 0;
    Kmtrx_bc(1,1) = 1;
    % Implement the essential b.c. u=ub at x=1.
    rload(:,1) =rload(:,1) - Kmtrx(:,npn)*ub;
    rload(npn,1) = ub;
    Kmtrx_bc(:,npn) = 0;
    Kmtrx_bc(npn,:) = 0;
    Kmtrx_bc(npn,npn) = 1;
end     % end function bnd_conds
