function [Cv_ice, dg_dtheta, D_s, k_s, usg] = ...
         snowpack_props_tc(npoin, phi_ha, phi_i, theta)
% This function computes snowpack properties of:
%     * thermal conductivity for snow
%     * effective diffusion coefficient of snow
%     * internal energy due to phase change from ice to water vapor
%     * specific heat of ice at constant volume
%     * derivative of water vapor density w.r.t. temperature
%              d(gamma_v)/d(theta)
%
% Base Units: (kilograms, meters, seconds, Kelvin, Newtons)
%
% ********************     Variable Definitions     ********************
% dg_dte       nodal value of d(gamma_v)/d(theta)
% dg_dtheta    nodal array of dg_dte
% Dva          temperature corrected value of the diffusion
%                      coefficient of water vapor in air
% D_s          effective mass diffusion coefficient for snow: 
%                      dimension(npoin)
% gamma_v      density of humid air
% k_air        thermal conductivity of air: (W/(m-K)) 
% k_ice        thermal conductivity of ice  (W/(m-K)) 
% k_s          thermal conductivity of snow: dimension(npoin)
% keff_s       effective thermal conductivty w mass diffusion effects:
%                      dimension(npoin)
% k_lam        thermal conductivity of lamellae microstructure 
%                      at a specific node
% k_pore       thermal conductivity of pore microstructure 
%                      at a specific node
% npoin        number of node points in 1-D spatial mesh
% phi_ha       humid air volume fraction: dimension(npoin)
% phi_i        ice volume fraction: dimension(npoin)
% te           temperature at a specific node (Kelvin)
% theta        temperature (Kelvin): dimension(npoin)
% usg          internal energy due to phase change from ice to water vapor:
%                     dimension(npoin)
% usg_te       internal energy due to phase change from ice to water vapor
%                     at a specific node
% v_ha         volume fraction of humid air at a specific node
% v_i          volume fraction of ice at a specific node
%
% **********************     End of Variables     **********************
%
% Initialize arrays for speed
Cv_ice = zeros(size(phi_i));
D_s = zeros(size(phi_i));
dg_dtheta = zeros(size(phi_i));
keff_s = zeros(size(phi_i));
k_s = zeros(size(phi_i));
usg = zeros(size(phi_i));
%
% Loop over the node points
for ipn = 1:npoin
    te = theta(ipn);
    v_i = phi_i(ipn);
    v_ha = phi_ha(ipn);
    [Cv_ice_te, dg_dte, Dva, k_air, k_ice, usg_te] = thermal_props(te);
    % Compute the thermal conductivity of the pore microstructure
    k_pore = v_i*k_ice + v_ha*k_air;
    %
    % Compute the thermal conductivity of the lamellae microstructure
    k_lam = k_ice*k_air/(v_i*(k_air+usg_te*Dva*dg_dte)+v_ha*k_ice);
    %
    % Compute the thermal conductivity of snow
    k_s(ipn,1) = v_i*k_pore + v_ha*k_lam;
    %
    % Compute the effective diffusion coefficint of water vapor in snow
    D_s(ipn,1) = v_i*v_ha*Dva...
        + v_ha*(k_ice*Dva)/(v_i*(k_air+usg_te*Dva*dg_dte)+v_ha*k_ice);
    %
    % Compute the diffusion enhanced effective thermal conductivity of
    % snow
    keff_s(ipn,1)=k_s(ipn)+v_ha*usg_te*D_s(ipn)*dg_dte;
    % 
    % Store the nodal value of dg_dte, Cv_ice_te, usg_ice_te
    Cv_ice(ipn,1) = Cv_ice_te;
    dg_dtheta(ipn,1) = dg_dte;
    usg(ipn) = usg_te;
end
end         % End of function snowpack_props
%
%********************     function thermal_props     ********************
%
function [Cv_ice_te, dg_dte, Dva, k_air, k_ice, usg_te] = thermal_props(te)
% This fuction computes thermal properties for snow that are a function
% of temperature, including:
%     * thermal conductivity for ice
%     * thermal conductivity for air
%     * effective diffusion coefficient of water vapor in air
%     * water vapor density
%     * derivative of water vapor density w.r.t. temperature
%     * second derivative of water vapor density w.r.t. temperature 
%     * internal energy change for ice to water vapor
%
% Base Units: (kilograms, meters, seconds, Kelvin, Newtons)
%
% ********************     Variable Definitions     ********************
%
% Cv_ice_te    specific heat of ice at constant volume : (N-m)/(kg-K)
% dg_dte       derivative of water vapor density w.r.t. temperature
% dg_dte2      second derivative of water vapor density w.r.t. temperature
% Dva_ref      base value of mass diffusion coefficient of 
%                      water vapor in air     (m^2/s)
% Dva          temperature corrected value of the diffusion  
%                      coefficient of water vapor in air  (m^2/s)
% gamma_v      density of water vapor         (kg/m^3)
% k_ice        thermal conductivity of ice    (W/(m*K))
% k_air        thermal conductivity of air    (W/(m*K))
% R_v          gas constant for water vapor   (N-m/(kg-K))
% te           temperature (K)
% te_cent      temperature in centigrade (C)
% te_ref       reference temperature for correction of diffusion
%                      coefficient of water vapor in air (K)
% usg_te       internal energy due to phase change from ice to water vapor
%
% **********************     End of Variables     **********************
%
% Define some constants:
% c1 through c6 are used to define the saturated water vapor density as a
% function of temperature (Dorsey(1968); Foslien(1994) pg 43
c1 = -2445.5646;
c2 = 8.2312;
c3 = -1.667006e-2;
c4 = 1.20514e-5;
c5 = -6.757169;
c6 = 133.3224;
%
Dva_ref = 0.26e-4;     % Incropera and Dewitt(1990), pg A25  (m^2/s)
te_ref = 298.0;        % Reference temperature for Dva_ref (K)
R_v = 461.5;           % (N-m)/(kg-K)
%
Ln10=log(10.0);         % a term used repeatedly resulting from:
                        % log10(x)=ln(x)/ln(10)
%
% Compute the thermal conductivity of ice using experimental correlation of
% Fukusako(1990: Int. J, Thermophysics) valid for -173C to 0 C.
te_cent = te-273.15;      % Temperature in centigrade
k_ice = 1.16*(1.91-8.66e-3*te_cent+2.97e-5*te_cent^2);   % W/(m*C)
%
% Compute the thermal conductivity of air using a best fit from data
% provided in Incropera and Dewitt(1985). Based on linear interpolation.
if (te >= 200 && te < 250)  
    k_air = 0.0181 + 8.4e-5*(te-200);       % (W/m*K)
elseif (te >= 250 && te <= 273.15)
    k_air = 0.0223 + 8.e-5*(te-250);        % (W/m*K)
else
    disp('Temperature outside bounds of interpolation.')
    return      % exit the program
end
%
% Compute the specific heat of ice using a best fit from data
% provided in Cengel(1989). Based on linear interpolation.
if (te >= 200 && te < 240)  
    Cv_ice_te = 1560 + 7.5*(te-200) ;       % (N-m/kg-K)
elseif (te >= 240 && te <= 273.15)
    Cv_ice_te = 1860 + 7.576*(te-240);      % (N-m/kg*K)
else
    disp('Temperature outside bounds of interpolation.')
    return      % exit the program
end
% 
% Compute the correction on the water vapor diffusivity due to changes in
% temperature.
Dva = Dva_ref*(te/te_ref)^1.5;
%
% Compute the internal energy due to phase change from ice to water vapor
usg_te = (2626.11+1.31763*te-3.71584e-3*te^2)*1.e3;
%
% Compute the vapor density of water from Dorsey(1968)
gamma_v = (10.0^(c1/te+c2*log(te)/Ln10+c3*te+c4*te^2+c5))*c6/R_v/te;
%
% Compute the first and second derivative of the water vapor density w.r.t.
% temperature.  Derivatives computed via MATLAB symbolic package.
% First derivative:
dg_dte = gamma_v*Ln10*(-c1/te^2+c2/(te*Ln10)+c3+2*c4*te) - gamma_v/te;
%
% Second derivative:
% Define some intermediate variables: g1 & g2
g1 = 2*c4 + 2*c1/te^3 - c2/(te^2*Ln10);
g2 = c3 + 2*c4*te - c1/te^2 + c2/(te*Ln10);
d2g_dte2 = (2/te^2)*gamma_v + gamma_v*Ln10*g1 +gamma_v * Ln10^2*g2^2 ...
         - gamma_v*Ln10*g2*2/te;
end     % end function thermal_props


