function [] = tgm_main_fea
tic
clear variables
clf
clc
% Written by: Andrew C. Hansen
% University of Wyoming
% Email:hansen@uwyo.edu
% Fall 2014
%
% This program is the main driver for the tgm solution of Foslien(1994).
% Code specific for Case 1 of The Cryosphere submission:tc-2015-34
% March 2015.
%
% Solution times are on the order of 2-3 minutes.
%
% ********************     Variable Definitions     ********************
% c_hat       mass supply (kg m^-3 s^-1): dimension(npoin)
% c_hat_mg    mass supply (mg m^-3 s^-1): dimension(npoin)
% Cv_ice      specific heat of ice (N-m)/(kg-K): dimension(npoin)                     
% Cv_air      specific heat of air at constant volume--assumed constant
% gamma_air   density of dry air  (1.34 kg/m^3)--value for 263K
% gamma_ice   density of ice (917 kg/m^3)
% H           time step size (seconds)
% k_s         thermal conductivity of snow (W/(m-K)): dimension(npoin)
% Le          element length
% nelem       number of elements
% npoin       number of nodes
% nsteps      number of time steps
% phi_ha      volume fraction of humid air: dimension(npoin)
% phi_i       volume fraction of ice: dimension(npoin)
% rho_s       density of snow: dimension(npoin)
% theta       nodal temperature: dimension(npoin)
% theta_a     temperature boundary condition at ground level (x=0)
% theta_b     boundary condition at surface (x=1)--may be time dependent
% theta_c     nodal temperature in Centigrade: dimension(npoin)
% theta_mean  mean surface temperature
% theta_fluc  fluctuation surface temperature for diurnal modeling
% t_final     final time
% t_n         time at the nth time step (initially t_n = 0)
% theta_it    iterative value of theta at time t_n+1
% theta_n1    nodal temperatures at time t_n+1
% usg         latent heat of sublimation of ice: dimension(npoin)
% x           spatial coordinate: dimension(npoin)
% **********************     End of variables     *************************
%
% Define the density of ice and the density of dry air
gamma_ice = 917.0;       % kg/m^3
gamma_air = 1.34;        % kg/m^3 value based on theta = 263 K
%
% Define the specific heat of dry air at constant volume
Cv_air = 716.5;        % value based on theta = 263 K (Cengel, 1989)
                       % units (N-m)/(kg-K)
% The specific heat of ice is calculated as a function of temperature
% using: "function snowpack_props"
%
% ***************     MODEL INPUT PARAMETERS     **************************
        nelem = 50;                   % Number of elements(3-node quadratic) 
        npoin = 2*nelem + 1;          % Number of points
        nsteps = 2900;                % Number of time steps
        H = 300;                      % step size (seconds)
        n_iter = 5;                   % Number of nonlinear iterations
        nplot = [288 1440 2880];      % Load steps to plot
        theta = 273 * ones(npoin,1);  % Initial temperature through snowpack
        % Surface temperature (theta_b) is specified within the load step 
        % to accomodate for time dependent surface conditions.  
        % Key parameters for surface boundary condition to follow:
        theta_mean = 253;
        theta_fluc = 0;
%
% Spatial parameters
Le = 1/nelem;               % Element length
x = linspace(0,1,npoin)';   % Node locations
%
% Time parameters
t_n= 0.0;                   % Initial time set to zero
t_final= nsteps * H;
%
% Initialize plot parameter
iplot = 1;                  
% Define initial conditions for the density of snow in terms of critical
% values where density changes.  General snowpack density is 240 kg/m^3 with
% an ice crust of 600 kg/m^3 followed by fresh snow at 120 kg/m^3. 
% The ground is also assumed to be solid ice.
x_crit = [0 0.04 0.64 0.72 0.78 0.86 1.0];
rho_s_crit=[917 240 240 600 600 120 120];
%
% Temperature parameters                             
theta_it = theta;     % Iterative value of temp. initially set to theta
theta_a = 273;        % b.c. @ x=0: The snow temperature at ground is 
                      % taken as freezing.
% ************************************************************************
%
% Initialize plot data
theta_c = zeros(length(x),length(nplot));         % temperature 
c_hat_mg = zeros(length(x),length(nplot));        % deposition 
dthetac_dx = zeros(length(x),length(nplot));      % temperature gradient
%
% Compute the volume fractions of ice and humid air
rho_s = interp1(x_crit,rho_s_crit,x);
phi_i = rho_s / gamma_ice;
phi_ha = 1.0 - phi_i;
%
figure(1)                            % snow density profile
   plot(x,rho_s,'LineWidth',2)
   grid on
   xlabel('Snow Height (m): 0=ground')
   ylabel('Snow Density (kg m^{-3})')
   title('Snow Density Profile')
   axis([0 1 0 1000])
%
% Initialize the mass supply to zero as an initial condition
c_hat = zeros(size(theta));
%
% Enter the time stepping routine
for istep = 1:nsteps;
    % Specify time at t_n+1
    t_n1 = t_n + H;
    % *******************************************************************
    % Specify the surface temperature
    theta_b = theta_mean - theta_fluc*sin(2*pi*t_n1/(24*3600));
    % *******************************************************************
    %
    % Compute the snowpack properties at the current increment
    [Cv_ice, dg_dtheta, D_s, k_s, usg] = ...
                      snowpack_props_tc(npoin, phi_ha, phi_i, theta);
    %                    
    % Define the parameters that are independent of nonlinear iteration 
    % to send to the solver of the diffusion equation given by:
    %
    %       (p u')' - q u + f - m u_t = 0                 0 < x < 1
    %
    % For the thermal/mass transfer problem
    % q = 0.0
    % p = k_s
    % m = phi_i*gamma_ice*Cv_ice + phi_ha*gamma_ha*Cv_air
    q = zeros(size(theta));
    p = k_s;
    m = phi_i*gamma_ice.*Cv_ice + phi_ha*gamma_air*Cv_air;
    %
    % Enter the nonlinear solver
    for iter = 0:n_iter
        % Compute the mass supply before entering the integration routine
        c_hat_n1 = ...
            mass_supply(dg_dtheta, D_s, H, Le, npoin, phi_ha, theta, theta_it);
        % 
        % f = 0.5*(u_sg*c_hat + usg*c_hat_n1)
        %          f is defined at time t_n and t_n1 based on the 
        %          Crank-Nicholson finite difference solution 
        f = 0.5*(usg.*c_hat + usg.*c_hat_n1);
        % Now enter the solver
        theta_it = ...
            diffusion_tc(H, f, m, p, q, theta, iter, nelem, npoin, Le, theta_a,theta_b);
    end
    %   
    % Update the temperature
    theta_n1 = theta_it;
    %
    % Compute the mass supply for the converged solution
    c_hat_n1 = ...
            mass_supply(dg_dtheta, D_s, H, Le, npoin, phi_ha, theta, theta_it);
    % Now update tn+1 results to the current results
    c_hat = c_hat_n1;
    t_n = t_n1;
    theta = theta_n1;
    theta_it = theta;
    % Update the volume fraction of ice by integrating the mass supply
    phi_i = phi_i + c_hat_n1*H/gamma_ice;
    phi_ha = 1.0 - phi_i;
    if iplot <= 3
        if istep == nplot(iplot)
            % Store the desired output with appropriate units
            theta_c(:,iplot)=theta-273;         % Conversion to Centigrade
            c_hat_mg(:,iplot) = c_hat*1.e6;     % Conversion to mg/(m^3-s)
            %
            % Compute the temperature gradient
            dthetac_dx(1,iplot) = (-3*theta(1) + 4*theta(2) - theta(3))/Le;
            dthetac_dx(npoin,iplot) = (theta(npoin-2) - 4*theta(npoin-1)...
                + 3*theta(npoin))/Le;
            for ipn = 2:npoin-1
                dthetac_dx(ipn,iplot) = (theta(ipn+1) - theta(ipn-1))/Le;
            end
            iplot = iplot + 1;
        end       
    end
end
figure(2)
    plot(x,theta_c(:,1),'b',x,theta_c(:,2),'r',...
        x,theta_c(:,3),'k','LineWidth',2)
    xlabel('Snow Height (m): 0=ground')
    ylabel('Temperature ( ^oC)')
    title('Temperature Profile')
    grid on
    legend('1 Day','10 Days','Location','SouthWest')
figure(3)
    plot(x,c_hat_mg(:,1),'b',...
        x,c_hat_mg(:,2),'r',...
        x,c_hat_mg(:,3),'k','LineWidth',2)
    xlabel('Snow Height (m): 0=ground')
    ylabel('Mass Supply (mg m^{-3} s^{-1})')
    title('Condensation Rate')
    grid on
    legend('1 Day','5 Days','10 Days','Location','SouthWest')
figure(4)
    plot(x,dthetac_dx(:,1),'b',...
         x,dthetac_dx(:,2),'r',...
         x,dthetac_dx(:,3),'k','LineWidth',2)
    xlabel('Snow Height (m): 0=ground')
    ylabel('Temperature Gradient ( ^oC m^{-1})')
    title('Temperature Gradient Profile')
    grid on
    legend('1 Day','5 Days','10 Days','Location','SouthWest')
toc
end       % end function tgm_main
%
%
function[c_hat_n1]...
    = mass_supply(dg_dtheta, D_s, H, Le, npn, phi_ha, theta, theta_it)
%     
% This function computes the mass supply following eqn. (72)
% of Hansen(2015).
%
% c_hat = d(D_s * dg_dtheta * dtheta_dx)/dx
%       - phi_ha * dg_dtheta * dtheta_dt
% 
% Spatial derivatives are evaluted at t_n+1--use t_n when iter = 0.
% At node n=1 and n=npoin we evaluate spatial derivatives using the 
% formula for quadratic interpolation of the element.  In the interior, we
% use a central finite difference approximation.
%
% Initialize dtheta_dx and c_hat_n1 for speed
c_hat_n1 = zeros(size(theta));
dtheta_dx = zeros(size(theta));
%
dtheta_dx(1) = (-3*theta_it(1) + 4*theta_it(2) - theta_it(3))/Le;
dtheta_dx(npn) = (theta_it(npn-2) - 4*theta_it(npn-1) + 3*theta_it(npn))/Le;
for ipn = 2:npn-1
    dtheta_dx(ipn) = (theta_it(ipn+1) - theta_it(ipn-1))/Le;
end
flux = D_s.*dg_dtheta.*dtheta_dx;
dtheta_dt = (theta_it - theta)/H;
vapor_rate = phi_ha.*dg_dtheta.*dtheta_dt;
%
% Compute the mass supply
c_hat_n1(1) = (-3*flux(1) + 4*flux(2) - flux(3))/Le - vapor_rate(1);
c_hat_n1(npn) = (flux(npn-2)-4*flux(npn-1)+3*flux(npn))/Le...
    - vapor_rate(npn);
for ipn = 2:npn-1
    c_hat_n1(ipn) = (flux(ipn+1) - flux(ipn-1))/Le - vapor_rate(ipn);
end
end %end function mass supply